/**********************************************************************
*  FileName:  schost.c                         
*
*  This program will determine if the Host is the Server or an
*   authorized remote client and return an appropriate RC            
*
*   Licensed Materials - Property of IBM                               
*                                                                      
*   (C) COPYRIGHT International Business Machines Corp. 1996           
*    All Rights Reserved.                                              
*                                                                      
*   US Government Users Restricted Rights - Use, duplication or        
*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.  
*                                                                      
*                    ------------------------------------              
*                                                                      
*               NOTICE TO USERS OF THE SOURCE CODE EXAMPLES            
*                                                                      
*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THE SOURCE 
*  CODE EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, 
*  "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,
*  INCLUDING, BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF 
*  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE 
*  RISK AS TO THE QUALITY AND PERFORMANCE OF THE SOURCE CODE EXAMPLES,
*  BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, IS WITH YOU.  SHOULD 
*  ANY PART OF THE SOURCE CODE EXAMPLES PROVE DEFECTIVE, YOU (AND NOT 
*  IBM) ASSUME THE ENTIRE COST IF ALL NECESSARY SERVICING, REPAIR OR 
*  CORRECTION.                                      
*                                                                      
**********************************************************************/

/*--------------------------------------------------------------
 |   Program includes                                          |
 --------------------------------------------------------------*/
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>


/*--------------------------------------------------------------
 |   Code starts here                                          |
 --------------------------------------------------------------*/
main(int argc, char *argv[])
{
  struct hostent *hostp;
  char  *outmsg = (char*) malloc(sizeof(char)*8192);
  char *pipeName = getenv("DTWPIPE");
  int count,
      pipeHandle,
      rc=0,
      retRC=1;

  if /* Check validate the number of arguments */
    (argc >= 4) {

    if /* Obtain the host name based on argv[1] */
      ((hostp = gethostbyname(argv[1])) != NULL) {

      outmsg[0] = '\0';

      sprintf(outmsg, "%s",
              inet_ntoa(*((int *)*(hostp->h_addr_list))));
      if /* Determine if server is the remote address */
        (!strcmp(argv[2], outmsg) ) {
        retRC=0;
      }
    }

    if /* If retRC=1 then match has not been found */
      (retRC) {

      count = 3;
      while /* Determine if remote address is in the list */
        (count <= argc)
      {
        if (!strcmp(argv[count++], argv[2]) ) {
          retRC=0;
          break;
        }
      }
    }
      
    if /*  Open the pipe using the name passed in the DTWPIPE   */
       /*   environment                                         */
      ((pipeHandle = open(pipeName,O_WRONLY)) != NULL) {

      /*--------------------------------------------------------------
       |  Write results to buffer                                    |
       --------------------------------------------------------------*/
      outmsg[0] = '\0';

      sprintf(outmsg,
              "RETURN_CODE=\"%d\"\n ",
              retRC);

      /*--------------------------------------------------------------
       |   Write buffer to pipe                                      |
       --------------------------------------------------------------*/
      count=write(pipeHandle,
                  outmsg,
                  strlen(outmsg));
      if (errno) {
        printf("schost Error:  write failed %d\n",errno);
        rc = -1;
      }
      else if (count != strlen(outmsg) || errno) {
        printf("schost Error:  write failed (%d&#60;&#62;%d)\n",
               count,
               strlen(outmsg));
        rc = -1;
      }

      /*--------------------------------------------------------------
       |   Close the pipe.                                           |
       --------------------------------------------------------------*/
      close(pipeHandle);

    } else {
      printf("schost Error:  open failed %d\n",errno);
    }
  } else {
    printf("schost Error:  Not enough args were passed (%d)\n",argc);
    rc=-1;
  }
  exit(rc);
}
