/*****************************************************************************
*  File Name: scshfile.cmd                                                   *
*                                                                            *
*  Description:  This program provides the algorithm to display macros,      *
*                 includes and execs with CMD extension as WEB Pages         *
*                                                                            *
*  Inputs:   file, username, macroPath, fAction, VNames, VNData, VField      *
*                                                                            *
*  Outputs:  N/A                                                             *
*                                                                            *
*  Returns:  N/A                                                             *
*                                                                            *
*                    ------------------------------------                    *
*                                                                            *
*   Licensed Materials - Property of IBM                                     *
*                                                                            *
*   (C) COPYRIGHT International Business Machines Corp. 1996                 *
*    All Rights Reserved.                                                    *
*                                                                            *
*   US Government Users Restricted Rights - Use, duplication or              *
*   disclosure restricted by GSA ADP Schedule Contract with IBM Corp.        *
*                                                                            *
*                    ------------------------------------                    *
*                                                                            *
*               NOTICE TO USERS OF THE SOURCE CODE EXAMPLES                  *
*                                                                            *
*  INTERNATIONAL BUSINESS MACHINES CORPORATION PROVIDES THE SOURCE CODE      *
*  EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, "AS IS" WITHOUT    *
*  WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT     *
*  LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A    *
*  PARTICULAR PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE    *
*  OF THE SOURCE CODE EXAMPLES, BOTH INDIVIDUALLY AND AS ONE OR MORE GROUPS, *
*  IS WITH YOU.  SHOULD ANY PART OF THE SOURCE CODE EXAMPLES PROVE           *
*  DEFECTIVE, YOU (AND NOT IBM) ASSUME THE ENTIRE COST IF ALL NECESSARY      *
*  SERVICING, REPAIR OR CORRECTION.                                          *
*                                                                            *
******************************************************************************/
trace off

env='environment'

LT = '&#60;' 
GT = '&#62;'

pInfo=Value('PATH_INFO',,env)

Parse Value Reverse(pInfo) With section '/' macroName '/'
showfile=macroPath||Reverse(macroName)||'/'||Reverse(section)

debugLine=17

Call GetINIpaths

filename=SysSearchPath(macroPath,Word(Translate(file,' ',','),1))
If filename='' Then 
 Do
  filename=SysSearchPath(includePath,Word(Translate(file,' ',','),1)) 
  If filename='' Then
   Do
    filename=SysSearchPath(execPath,Word(Translate(file,' ',','),1))
    /* If blank create a dummy file so as not to default to STDIN **/
    If filename='' Then filename='F$'||Random()
   End
 End


Parse Value Reverse(filename) With pfile '/' .
pfile = Reverse(pfile)

Say "<html>"
Say "<title>Display of file " pfile "</title>"
Say "<body>"
Say "<h1>Source display of " pfile "</h1>"

Say "<hr> <p>"
Say "<pre>"

j=0
execLine=''

Do While Lines(filename) > 0
 fspec=''
 j=j+1
 macroLine=Linein(filename)
 If Pos('%INCLUDE', Translate(macroLine))=1 &,
    Pos('SCCONF.HTI',Translate(macroLine))=0 Then 
  Do
   path=Strip(Strip(Strip(Substr(macroLine,Length("%INCLUDE")+1)),,'"'),,"'")
   fspec=SysSearchPath(includePath,path)
   If Strip(fspec)<>'' Then fspec=path
  End
 Else If Pos('%EXEC', Translate(macroLine))>1 Then execLine=macroLine

 If execLine<>'' Then
  Do
   If Pos('.CMD', Translate(macroLine))>0 Then 
    Do
     Parse Var macroLine '{' cmdLine '.' fileEXT .
     cmdLine=Strip(Translate(cmdLine,'/','\'))||'.'||fileEXT
     path=cmdLine
     fspec=SysSearchPath(execPath,path)
     If Strip(fspec)<>'' Then fspec=path
     execLine=''
    End
   Else If Pos('.EXE', Translate(macroLine))>0 Then
    Do
     Parse Var macroLine '{' cmdLine '.' fileEXT .
     cmdLine=Strip(Translate(cmdLine,'/','\'))
     Parse Value Reverse(cmdline) With cmdline ')' cmdline2
     If cmdline='' Then path=Reverse(cmdline2)
                   Else path=Reverse(cmdline)
     fspec=SysSearchPath(execPath,path||'.'||fileEXT)
     Cext='.CPP'
     fview=SysSearchPath(execPath,path||Cext)
     If Strip(fview)='' Then 
      Do
       Cext='.cpp'
       fview=SysSearchPath(execPath,path||Cext)
       If Strip(fview)='' Then 
        Do
         Cext='.C'
         fview=SysSearchPath(execPath,path||Cext)
         If Strip(fview)='' Then 
          Do
           Cext='.c'
           fview=SysSearchPath(execPath,path||Cext)
          End
        End 
      End
     If Strip(fspec)<>'' & Strip(fview)<>'' Then
      Do
       fspec=path||'.'||fileEXT
       fview=path||Cext
      End
     execLine=''
    End
   Else execLine=execLine||' '||macroLine
  End 

 Say DisplayLine(Right(j,3,'0') macroLine,fspec)

End /* Do While */

Say "</pre>"
Say "<hr>"

Call CreateReturn

Say "</body>"
Say "</html>"

Exit


/*****************************************************************************
*  Subroutine: addESCseq                                                     *
*                                                                            *
*  Description:  Converts URL disallowed characters TAB(09), SPACE(20),      *
*                 "(22), <(3C), >(3E), [(5B), \(5C), ](5D), ^(5E), `(60),    *
*                 {(7B, |(7C), }(7D), ~(7E) into valid URL encoded values    *
*                                                                            *
******************************************************************************/
addESCseq:
  Procedure
  Parse Arg a
  nURLchar='0920222325262F3A3B3C3D3E3F405B5C5D5E607B7C7D7E'x
  n=Verify(a,nURLchar,"M")                    /** find first instance        **/
  Do While n>0
   newChar = '&#'||C2d(Substr(a,n,1))';'      /** get URL encoded char       **/
   a=Delstr(Insert(newChar,a,n),n,1)          /** replace disallowed char    **/
   n=Verify(a,nURLchar,"M",n+Length(newChar)) /** move beyond inserted char  **/
                                              /**  and find the next instance**/
  End
Return a

/*****************************************************************************
*  Subroutine: CreateReturn                                                  *
*                                                                            *
*  Description:  Creates the return button (form)                            *
*                                                                            *
******************************************************************************/
CreateReturn:
 If Words(Translate(file,' ',','))=1 Then
  Do
   Say '<form method="post" action="'fAction'">'
   W_VNames=Translate(VNames,' ',',')
   W_VNData=Strip(VNData,,',')||','
   W_VField=Translate(VField,' ',',')
   N_VNames=Words(W_VNames)
   Do i=1 to N_VNames
    N_VField=Word(W_VField,i)
    V_VData=''
    Do j=1 to N_VField 
     Parse Var W_VNData VData ',' W_VNData
     V_VData=V_VData||VData||',' 
    End
    V_VData=addESCseq(Strip(V_VData,,','))
    Say '<input type="hidden" name="'Word(W_VNames,i)'" value="'V_VData'">'
   End
  End
 Else 
  Do
   Say '<form method="post" action="'showfile'">'
   filename=Translate(Subword(Translate(file,' ',','),2),',',' ')
   Say '<input type="hidden" name="VNames"   value="'addESCseq(VNames)'">'
   Say '<input type="hidden" name="VNData"   value="'addESCseq(VNData)'">'
   Say '<input type="hidden" name="VField"   value="'VField'">'
   Say '<input type="hidden" name="filename" value="'filename'">'
   Say '<input type="hidden" name="pReturn"  value="'fAction'">'
  End

 Say '<input type="hidden" name="usrname"  value="'addESCseq(username)'">'
 Say '<input TYPE=SUBMIT value="Return">'
 Say '</form>'
Return

/*****************************************************************************
*  Subroutine: DisplayLine                                                   *
*                                                                            *
*  Description:  Creates a button to allow the source display of includes    *
*                 and execs found within a macro or includes                 *
*                                                                            *
******************************************************************************/
DisplayLine:
 Parse Arg dLine, special
 dLine=FindReplace(dline,'<',LT) 
 dLine=FindReplace(dline,'>',GT) 
 If special <> '' Then 
  Do
   dform= '<Form method="post" action="'showfile'">'
   dform= dform||Substr(dline,1,Pos(path,dline)-1)
   dform= dform||'<input type="hidden" name="VNames"   value="'addESCseq(VNames)'">'
   dform= dform||'<input type="hidden" name="VNData"   value="'addESCseq(VNData)'">'
   dform= dform||'<input type="hidden" name="VField"   value="'VField'">'
   dform= dform||'<input type="hidden" name="filename" value="'special||','||file'">'
   dform= dform||'<input type="hidden" name="usrname"  value="'addESCseq(username)'">'
   dform= dform||'<input type="hidden" name="pReturn"  value="'fAction'">'
   dform= dform||'<input type="SUBMIT" value="'path'">'
   dform= dform||Substr(dline,Pos(path,dline)+Length(path))
   dform= dform||'</form>'
   dform= dform||''
  End 
 Else dform=dLine
Return dform


/*****************************************************************************
*  Subroutine: FindReplace                                                   *
*                                                                            *
*  Description:  Given a variable to find it replaces it with another        *
*                                                                            *
******************************************************************************/
FindReplace:
 Procedure Expose j debugLine
 Parse Arg lineVar, findVar, replaceVar
x=0
 posVar=Pos(findVar,lineVar)
 Do While posVar>0
  temp=Delstr(lineVar,posVar,1)
  temp2=insert(replaceVar,temp)
  lineVar=DelStr(Insert(replaceVar,lineVar,posVar),posVar,1)
  posVar=Pos(findVar,lineVar,posVar)

If x>30 then 
 Do
  SAY "<H2>ERROR: During 'View' of source code line="j
  SAY "temp= "temp
  SAY "temp2= "temp2
  SAY "lineVar= "lineVar
  SAY "posVar= "posVar
  say "length= "Length(lineVar)
  leave
 End
Else x=x+1

 End
Return lineVar


/*****************************************************************************
*  Subroutine: GetINIPaths                                                   *
*                                                                            *
*  Description:  Get all the macro, include and exec paths from DB2WWW.INI   *
*                                                                            *
******************************************************************************/
GetINIpaths:
 pTrans=Value('PATH_TRANSLATED',,env)
 posInfo=Pos(pInfo,pTrans)
 If posInfo>0 Then INIPath=Translate(Substr(pTrans,1,posInfo-1),'/','\')
 INIfile=INIpath||'/db2www.ini'
 Do While Lines(INIfile)
  trueFlag=1
  Parse Value Linein(INIfile) With cmd path
  Select 
   When cmd = 'MACRO_PATH'   Then macroPath=SetPath(path)
   When cmd = 'INCLUDE_PATH' Then includePath=SetPath(path)
   When cmd = 'EXEC_PATH'    Then execPath=SetPath(path)
   Otherwise NOP
  End /* Select */
 End
Return


/*****************************************************************************
*  Subroutine: SetPath                                                       *
*                                                                            *
*  Description:  Set an environment variable with the given path             *
*                                                                            *
******************************************************************************/
SetPath:
 Procedure Expose env
 Parse Arg sPath
 Do Forever
  setName='MP'||Random()
  If Value(setName,,env)='' Then
   Do
    Call Value setName,spath,env
    Leave
   End
 End
Return setName


/*****************************************************************************
*  Subroutine: SysSearchPath                                                 *
*                                                                            *
*  Description:  Searches the specified path for the specified file and      *
*               an returns the full filespec of the file if found; otherwise *
*               it returns a blank                                           *
*                                                                            *
******************************************************************************/
SysSearchPath:
 Procedure Expose env
 Parse Arg sPath, fileName
 sPath = Value(sPath,,env)
 sPath=Translate(sPath," /",";\")
 Do i=1 to Words(sPath)
  dirFile = Strip(Word(sPath,i),'T','/')||'/'||fileName
 /* determine if file exists   */
 /* '/usr/bin/test -f 'dirFile */
  If Stream(dirFile,'C','open read') = "READY:" Then
   Do
    rc=Stream(dirFile,'C','close')
    Return dirFile
   End
 End
Return ""
